#include "system.h"

WORD sw2HoldTime;
WORD sw3HoldTime;
BYTE sw2Pressed;
BYTE sw3Pressed;


/****************************************************************************
  :
    void EnableSW2(void)
  
        2
  ***************************************************************************/
void EnableSW2(void)
{
    SW2_TRIS = 1;
    SW2_PULLUP = 1;     

    _INT0EP = 1;
    _INT0IF = 0;
    _INT0IP = 4;
    _INT0IE = 1;
}


/****************************************************************************
  :
    void DisableSW2(void)

        2
  ***************************************************************************/
void DisableSW2(void)
{
    _INT0IE = 0;
    _INT0IF = 0;

    SW2_TRIS = 0;
    SW2_PULLUP = 0;
}


/****************************************************************************
  :
    void EnableSW3(void)

        3
  ***************************************************************************/
void EnableSW3(void)
{
    SW3_TRIS = 1;       //sw3 input
    SW3_PULLUP = 1;     //Enable sw3 pull up

    _CNIF = 0;          //configure interrupt for sleep wakeup
    _CNIP = 4;
    _CN12IE = 1;
    _CNIE = 1;
}


/****************************************************************************
  :
    void DisableSW3(void)

        3s
  ***************************************************************************/
void DisableSW3(void)
{
    _CN12IE = 0;
    _CNIE = 0;
    _CNIF = 0;

    SW3_TRIS = 0;
    SW3_PULLUP = 0;             
}


/****************************************************************************
  :
    void ProcessButtons(void)
  
   
  ***************************************************************************/
void ProcessButtons(void)
{

    _INT0IF = 0;    //   
    _INT0IE = 0;
    _CNIF = 0;
    _CNIE = 0;

    SW2_TRIS = 1;
    SW2_PULLUP = 1; // 
    SW3_TRIS = 1;
    SW3_PULLUP = 1; //  

    IdleUs(5);  // 

    sw2Pressed = SW2_PORT ^ 1;  //   2
    sw3Pressed = SW3_PORT ^ 1;  //   3

	#ifdef USE_BUTTON_HOLD
    sw2HoldTime = 0; //    
    sw3HoldTime = 0;

    while(SW3_PORT == 0 || SW2_PORT == 0)   //    
    {
        SW2_PULLUP = 0; //  
        SW3_PULLUP = 0; // 

        IdleMs(BUTTON_IDLEMS); // 

        SW2_PULLUP = 1; //  
        SW3_PULLUP = 1; // 
        IdleUs(5);

        if(SW2_PORT == 0)   //    
        {
            sw2HoldTime++;
        }

        if(SW3_PORT == 0)
        {
            sw3HoldTime++;
        }
    }
	#endif

    SW2_PULLUP = 0;
    SW3_PULLUP = 0; //  
}//end ProcessButtons


/****************************************************************************
  :
    void HandleButtons(void)

    

   SW2:         
   SW3:     / UART
  SW2  (2 ):    
  SW3  (2 ):    
   SW2  SW3  (2 ):       EEPROM

  
  SW3      DeepSleep.    SW3  DeepSleep
  ,  SW3,    SW2
  
   DeepSleep    
  ***************************************************************************/
void HandleButtons(void)
{

    #ifdef USE_PWRCTL
    mPWR_ON();  
    #endif

    if(tasks.bits.transmit)
    {
        _UxMD = 0;
        UARTInit();
    }

    if(sw2Pressed && !sw3Pressed)   //SW2 , SW3 
    {
	    potVal_prev = 0;		//     ( )
		tempVal = 0;	//    0,1
		vddVal = 0;		//     
  
        mSetLED2Tris(0); //  
        mSetLED2(0);

        if(tasks.bits.transmit)
        {
	        UARTPrintString("Wake up!\n\r");
        }

        tasks.bits.sample = 1;   //  

		

        #ifdef USE_BUTTON_HOLD
        //  SW2  
        if(sw2HoldTime >= (BUTTON_HOLDTIME/BUTTON_IDLEMS))
        {
            tasks.bits.mode += 1;           //  
			
            #ifdef USE_CAPTOUCH
            if(tasks.bits.mode > MODE_POT)  
            {
                if(lpMode > LP_SLEEP)       
                {
                    lpMode = LP_SLEEP;      

                    if(tasks.bits.transmit)
                    {
                        UARTPrintString("Cap Touch buttons not supported in Deep Sleep.\r\n");
                        UARTPrintString("Changing low power mode to Sleep.\r\n");
                    }
                }

                CapSenseTickInit(); 
            }
            else
            {
	            _T1MD = 1;         
                _T1IE = 0;
            }
            #endif

			#ifndef USE_CAPTOUCH
			if (tasks.bits.mode > MODE_ALL){
		        tasks.bits.mode = MODE_TEMP;
		    } 
            #endif
			
            if(tasks.bits.transmit)
            {
	            
                TransmitSampleModeChange(); //    
            }
        }
        #endif

        mSetLED2(1);              //  
        mSetLED2Tris(1);


    }
    else if(sw3Pressed) //SW3   SW2  SW3 
    {

        mSetLED3Tris(0);
        mSetLED3(0);

        #ifdef USE_BUTTON_HOLD
        if(sw2HoldTime >= (BUTTON_HOLDTIME/BUTTON_IDLEMS) &&   //   
           sw3HoldTime >= (BUTTON_HOLDTIME/BUTTON_IDLEMS))
        {
            InitI2C();  //  UART  I2C
            _UxMD = 0;
            UARTInit();

            // 
            UARTPrintString("Transmitting datalog:\n\r");
            TransmitEEPROM(0,eeAddress);
            UARTPrintString("Datalog Transmit done!\n\r");

        }
        else if(sw3HoldTime >= (BUTTON_HOLDTIME/BUTTON_IDLEMS)) //  SW3
        {
            lpMode++;  //   
            if(lpMode > LP_DPSLP)
            {
                lpMode = LP_IDLE;
				#ifdef USE_CAPTOUCH
                tasks.bits.mode = MODE_CAP;
                #endif
            }

            #ifdef USE_CAPTOUCH
            
            if((tasks.bits.mode > MODE_POT) && (lpMode > LP_SLEEP))
            {
                lpMode = LP_SLEEP;

                if(tasks.bits.transmit)
                {
            
                    UARTPrintString("Could not switch low power mode to Deep Sleep.\n\r");
                    UARTPrintString("Please switch sensor type first.\n\r") ;
                }
            }
            #endif

            if(tasks.bits.transmit)
            {
                TransmitLPModeChange(); 
            }
        }
        else    //SW3 
        {
            if(tasks.bits.transmit)
        	{
		        UARTPrintString("Data transmission is switched off\n\r");
	            UARTPrintString("Press SW3 to switch data transmission on\n\r");
	        }
            
            tasks.bits.transmit ^= 1;    // / UART

            if(tasks.bits.transmit) //
            {
                _UxMD = 0;
                UARTInit();
                UARTPrintString("UART DISPLAY Enabled.\n\r") ;
            }
        }
        #else
            tasks.bits.transmit ^= 1;  

            if(tasks.bits.transmit) 
            {
                _UxMD = 0;
                UARTInit();
                UARTPrintString("UART DISPLAY Enabled.\n\r") ;
            }
        #endif

        mSetLED3(1);
        mSetLED3Tris(1);

    }

    _UxMD = 1;              //
    #ifdef USE_PWRCTL
    mPWR_OFF();
    #endif

    tasks.bits.button = 0;  //clear button task flag

}//end HandleButtons


