#include "system.h"

WORD curRawData[NUM_CTMU_CH];   //    CTMU
WORD tripValue[NUM_CTMU_CH];    // 
WORD hystValue[NUM_CTMU_CH];    //  
WORD avg_delay[NUM_CTMU_CH];    //    
WORD immediateValue;            //  
WORD averageData[NUM_CTMU_CH];  // 
WORD smallAvg[NUM_CTMU_CH];     // 
int startupCount;               // 


/****************************************************************************
  :
    void InitCapSense(void)
    
   CTMU
  ***************************************************************************/
void InitCapSense(void)
{
    BYTE index; //   
    BYTE count;

    // 
    for (index=0; index < NUM_CTMU_CH; index++)
    {
        curRawData[index] = 0;
        averageData[index] = 0;
    }

    
    tripValue[0] = TRIPVAL;
    tripValue[1] = TRIPVAL;
    tripValue[2] = TRIPVAL;

    
    hystValue[0] = 0x3;
    hystValue[1] = 0x3;
    hystValue[2] = 0x3;

    
    startupCount = 32;

    for(count = 0;count < CAP_NUM_AVG;count ++)
    {
        GetCapVal();
    }

}


/****************************************************************************
  :
    void CapSenseTickInit(void)
     
  ***************************************************************************/
void CapSenseTickInit( void )
{
    EnableSOSC();           //   32
    _T1MD = 0;              //  Timer1

    TMR1 = 0;               //  Timer
    PR1 = TIMER1_PERIOD;    //  
    T1CON = 0x8022;         // 1:64 

    _T1IF = 0;              //     Timer1
    _T1IE = 1;              //   
}


/****************************************************************************
  Function:
    void ReadCTMU(int index)

    CTMU
  ***************************************************************************/
void ReadCTMU(int index)
{

    _CTMUMD = 0;        //Enable CTMU
    if(_ADC1MD)
    {
        _ADC1MD = 0;    //Enable ADC

        POT_AN = 0;     //reset AD1PCFG after clearing PMD
        TEMP_AN = 0;
        TEMP_TRIS = 1;  //TEMP IN
        POT_TRIS = 1;   //POT IN
    }

    CTMUCON = 0;
    _EDG2SEL = 0x3;     //edge2 src = CTED1
    _EDG1POL = 1;       //edge1 postive
    _EDG1SEL = 0x3;     //edge1 src = CTED1
    CTMUICON = 0x300;   //0.55uA

    AD1CON1 = 0x8000;   //adc on
    _CTMUEN = 1;        //CTMU on

    //Ground selected CTMU channel
    switch(index)
    {
        case 0:
            CT1_LAT = 0;                //ground CT1
            CT1_TRIS = 0;
            Nop();Nop();Nop();Nop();Nop();
            CT1_TRIS = 1;               //ct1 as analog input
            CT1_AN = 0;
            AD1CHS = CT1_CHANNEL;
            break;

        case 1:
            CT2_LAT = 0;                //ground CT2
            CT2_TRIS = 0;
            Nop();Nop();Nop();Nop();Nop();
            CT2_TRIS = 1;               //ct2 as analog input
            CT2_AN = 0;
            AD1CHS = CT2_CHANNEL;
            break;

        case 2:
            CT3_LAT = 0;                //ground CT3
            CT3_TRIS = 0;
            Nop();Nop();Nop();Nop();Nop();
            CT3_TRIS = 1;               //CT3 as analog input
            CT3_AN = 0;
            AD1CHS = CT3_CHANNEL;
            break;

        default:
            break;
    }

    //Drain any internal ADC charge
    _IDISSEN = 1;   //Drain any charge on the circuit
    Nop(); Nop(); Nop(); Nop(); Nop();
    _IDISSEN = 0;

    //Charge the capacitive sensor
    _AD1IF = 0;
    _SAMP = 1;      //Manually start the conversion

    _EDG2STAT = 0;  //Make sure edge2 is 0
    _EDG1STAT = 1;  //Set edge1 - Start Charge
    Nop();Nop();Nop();Nop();Nop();Nop();    //Delay for CTMU charge time
    _EDG1STAT = 0;  //Clear edge1 - Stop Charge

    _SAMP = 0;      //Manually Initiate an ADC conversion
    while(!_AD1IF); //Wait for the A/D conversion to finish

    immediateValue = ADC1BUF0;  //Read the value from the A/D conversion

    _SAMP = 0;      //Clear ADC status
    _AD1IF = 0;
    _DONE = 0;

    curRawData[index] = immediateValue; //curRawData array holds the most recent BIGVAL values

    _CTMUMD = 1;    //CTMU & ADC off to save power
    _ADC1MD = 1;

} //end ReadCTMU()


/****************************************************************************
  Function:
    void UpdateCTMU(int index)

       
  ***************************************************************************/
void UpdateCTMU(int index)
{

    //1. On power-up, reach steady-state readings
    //During power-up, system must be Initialized, Set average ea. pass = raw data
    if (startupCount > 0)
    {
        startupCount--;         //Decr. N # times to establish startup
        averageData[index] = curRawData[index]; //During start up time, Set Average each pass.
    }
    else
    {
        //Startup complete
        if(startupCount == 0)
        {
            startupCount = -1;
        }

        //2. Detect cap sense button press
        if (curRawData[index] < (averageData[index] - tripValue[index]))
        {
            switch(index)
            {
                case 0:
                    if(!pressedCT1)     //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT1 = 1;     //button is pressed
                    break;

                case 1:
                    if(!pressedCT2)     //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT2 = 1;      //button is pressed
                    break;

                case 2:
                    if(!pressedCT3)     //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT3 = 1;     //button is pressed
                    break;

                default:
                    break;
            }//end switch(index)
        }
        else    //3. If pressed criteria not reached, Set to not pressed
        {
            switch(index)
            {
                case 0:
                    if(pressedCT1)      //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT1 = 0;     //button is pressed
                    break;

                case 1:
                    if(pressedCT2)      //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT2 = 0;     //button is pressed
                    break;

                case 2:
                    if(pressedCT3)      //log state changes only
                    {
                        tasks.bits.store = 1;
                    }

                    pressedCT3 = 0;     //button is pressed
                    break;

                default:
                    break;
            }//end switch(index)

        }

         //4. Quick-release for a released button.
        //When button is released, reset average to new max value
        if (curRawData[index]  > averageData[index])
        {
            averageData[index] = curRawData[index];
        }

        //5. Average in the new value
        //Average (all buttons) each NUM_AVG times through the algorithm
        if(avg_delay[index] < CAP_NUM_AVG)
        {
            avg_delay[index]++;
        }
        else
        {
            avg_delay[index] = 0;
        }

        if(avg_delay[index] == CAP_NUM_AVG)     //Average in raw value.
        {
            smallAvg[index] = averageData[index] / CAP_NUM_AVG;
            averageData[index] = averageData[index] + ((curRawData[index] / CAP_NUM_AVG) - smallAvg[index]);
            //Note: This is not a true average.  It is:  New Average = Last Average + (New Value/8 - Last Average/8)
        }

    }//end if(startupCount > 0)

}//end UpdateCTMU(..)


/****************************************************************************
  Function:
    void GetDiodeTemp(int Index)

  Summary:
    Measure temperature using CTMU and junction diode.

  Description:
    Measures current temperature by measuring forward voltage of diode with
    constant current applied.

  Precondition:
    None.

  Parameters:
    int index - ADC channel to Sample for diode temp measurement

  Returns:
    None.
  ***************************************************************************/
void GetDiodeTemp(int index)
{
    DWORD total = 0;
    int chrd;
    int zvar;
    int diodeValue;

    _CTMUMD = 0;        //clear PMD bits
    if(_ADC1MD)
    {
        _ADC1MD = 0;

        POT_AN = 0;     //reset AD1PCFG after clearing ADC PMD
        TEMP_AN = 0;
        TEMP_TRIS = 1;
        POT_TRIS = 1;
    }

    CTMUCON = 0;
    _EDG2SEL = 0x3;     //edge2 src = CTED1
    _EDG1POL = 1;       //edge1 postive
    _EDG1SEL = 0x3;     //edge1 src = CTED1

    CTMUICON = 0;
    _IRNG = 0x03;       //0.55uA

    AD1CON1 = 0x8000;   //adc on
    AD1CHS = index;     //select A/D channel

    _CTMUEN = 1;        //Enable CTMU

    //Take DIODE_NUM_AVG readings of diode AN channel
    for(chrd=0; chrd<DIODE_NUM_AVG; chrd++)
    {
        TEMP_AN = 0;
        TEMP_LAT = 0;   //GROUND AN channel
        TEMP_TRIS = 0;
        Nop(); Nop(); Nop(); Nop(); Nop();

        TEMP_AN = 1;    //switch back to AN input
        TEMP_TRIS = 1;

        _IDISSEN = 1;  //Drain any charge on the ADC circuit
        Nop(); Nop(); Nop(); Nop(); Nop();
        _IDISSEN = 0;

        _AD1IF = 0;
        _SAMP = 1;      //Manually start the conversion
        _EDG2STAT = 0;  //Make sure edge2 is 0
        _EDG1STAT = 1;  //Set edge1 - Start Charge

        for (zvar = 0; zvar < 5; zvar++);    //Delay for CTMU charge time

        _EDG1STAT = 0;  //Clear edge1 - Stop Charge
         _SAMP = 0;     //manually Initiate an ADC conversion
        while(!_AD1IF); //Wait for the A/D conversion to finish

        _SAMP = 0;      //clear ADC status
        _AD1IF = 0;
        _AD1IE = 0;

        diodeValue  = ADC1BUF0; //Read the value from the A/D conversion

        total = total + diodeValue; //sum sequential reads
    }

    tempVal = total/DIODE_NUM_AVG;    //Store resulting average to tempVal

	#ifdef OPTIMIZE_B
    _CTMUMD = 1;        //disable ADC and CTMU for power savings
    _ADC1MD = 1;
	#endif
}//end GetDiodeTemp(..)



