#include "system.h"


/****************************************************************************
  :
    void InitI2C(void)
   I2C
  ***************************************************************************/
void InitI2C(void)
{
    _I2C1MD = 0;            //  I2C  Timer2
    _T2MD = 0;

    I2C1CON = 0x8000;       //  I2C
    I2C1BRG = I2CBRG_VAL;   //  
    _MI2C1P = 1;            //  
}


/****************************************************************************
  :
    void CloseI2C(void)

    I2C
  ***************************************************************************/
void CloseI2C(void)
{
    I2C1CON = 0;    //  I2C

    //   
    #ifdef OPTIMIZE_B
        _I2C1MD = 1;        //  I2C  Timer2    (PMD )
        _T2MD = 1;
    #endif
}


/****************************************************************************
  :
    void I2CWait(void)
    Idle   I2C
  ***************************************************************************/
int I2CWait(void)
{
    _MI2C1IF = 0;   //   
    _MI2C1IE = 1;

    T2IntOn();      
    while((!_T2IF) && (!_MI2C1IF))
    {
        Idle();
    }
    T2IntOff();

    _MI2C1IE = 0;   //  
    _MI2C1IF = 0;

    if(_T2IF)
    {
        _T2IF = 0;
        return -1;  
    }

    return 0;       
}


/****************************************************************************
  :
    void T2IntOn(void)

  Timer2    
  ***************************************************************************/
void T2IntOn(void)
{
    _T2MD = 0;
    TMR2 = 0x0000;
    PR2 = 0xFFFF;
    _T2IF = 0;
    _T2IP = 1;
    _T2IE = 1;
    T2CON = 0x8000;
}


/****************************************************************************
  :
    void T2IntOff(void)

  Timer2    
  ***************************************************************************/
void T2IntOff(void)
{
    T2CON = 0;
    _T2IE = 0;
    _T2IF = 0;
}


/****************************************************************************
  :
    BYTE writePacket(WORD addr, BYTE* packet, BYTE numBytes)

      EEPROM

  :
    WORD addr       -  EEPROM   
    BYTE* packet    -    
    BYTE numbytes   -    

  :
    BYTE -   
  ***************************************************************************/
BYTE writePacket(WORD addr, BYTE* packet, BYTE numBytes)
{
    int i;
    char test;

    //  -
    I2C1CONbits.SEN = 1;
    test = I2CWait();     //  

    //   
    I2C1TRN = EEPROM_ADDRESS | I2C_WRITE;
    test = I2CWait();     //  

    //   
    I2C1TRN = addr>>8;
    test = I2CWait();     //  
    I2C1TRN = addr;
    test = I2CWait();     //  

    //  
    for(i = 0; i<numBytes;i++)
    {
        if(i>0 && I2C1STATbits.ACKSTAT)
        {
            //NACK,reTransmit
            i--;
        }

        I2C1TRN = packet[i];
        test = I2CWait();     //  
    }

    // -
    I2C1CONbits.PEN = 1;
    test = I2CWait();     //  

    return 0;
}


/****************************************************************************
  :
    BYTE readPacket(WORD addr, BYTE* packet, BYTE numBytes)

    EEPROM
  ***************************************************************************/
BYTE readPacket(WORD addr,BYTE *packet, BYTE numBytes)
{
    int i;

    // -
    I2C1CONbits.SEN = 1;
    I2CWait();     //  

    //   
    I2C1TRN = EEPROM_ADDRESS | I2C_WRITE;
    I2CWait();     //  

    //   
    I2C1TRN = addr>>8;
    I2CWait();     //  
    I2C1TRN = addr;
    I2CWait();    //  


    // 
    I2C1CONbits.RSEN = 1;
    I2CWait();     //  

    //  
    I2C1TRN = EEPROM_ADDRESS | I2C_READ;
    I2CWait();    //  

    //  
    for(i = 0; i<numBytes;i++)
    {


        I2C1CONbits.RCEN = 1;
        I2CWait();     //  


        packet[i] = I2C1RCV;


        if(i==numBytes-1)
        {
            I2C1CONbits.ACKDT = 1;
        }
        else
        {
            I2C1CONbits.ACKDT = 0;
        }

        I2C1CONbits.ACKEN = 1;
        I2CWait();     //  
    }

    // -
    I2C1CONbits.PEN = 1;
    I2CWait();     //  
    return 0;
}


/****************************************************************************
  :
    BYTE writeWait()

      EEPROM
  ***************************************************************************/
BYTE writeWait()
{
    BYTE retVal;

    // -
    I2C1CONbits.SEN = 1;
    I2CWait();          


    I2C1TRN = EEPROM_ADDRESS | I2C_WRITE;
    I2CWait();          

    //0 = ACK  EEPROM,  
    //1 = NACK  EEPROM,  
    retVal = I2C1STATbits.ACKSTAT;  

    // 
    I2C1CONbits.PEN = 1;
    I2CWait();          

    return retVal;
}


/****************************************************************************
  :
    void StoreAddress(void)

      EEPROM
  
***************************************************************************/
void StoreAddress (void)
{
    BYTE datVal[2];

    datVal[0] = eeAddress;
    datVal[1] = eeAddress >>8;

    writePacket(0x0, datVal, 2);

    IdleMs(EEPROM_WRITE_WAIT);
}



/****************************************************************************
  :
    void GetAddress(void)

       EEPROM
  ***************************************************************************/
void GetAddress (void)
{
    BYTE dataVal[2];
    BYTE *pByte;


    pByte = dataVal;
    readPacket(0,pByte,0x02);


    eeAddress = 0xFFF0 & (dataVal[0] + (((WORD)dataVal[1])<<8));


    if(eeAddress<0x10 || eeAddress > 0x7FFF)
    {
        eeAddress = 0x10;
    }

}


/****************************************************************************
  :
    void GetTimestamp(WORD addr)

       EEPROM     RTCC

  :
    WORD addr -  EEPROM,    
***************************************************************************/
void GetTimestamp(WORD addr)
{
    BYTE dataVal[7];
    BYTE *pData;

 
    pData = dataVal;
    readPacket(addr,pData,0x7);


    rtcc.byte.year = *pData++;
    rtcc.byte.mon = *pData++;
    rtcc.byte.mday = *pData++;
    rtcc.byte.wday = *pData++;
    rtcc.byte.hour = *pData++;
    rtcc.byte.min = *pData++;
    rtcc.byte.sec = *pData++;

    UpdateRTCC();

}


/****************************************************************************
  :
    void StoreData(WORD addr, WORD data, BYTE type)
    
      EEPROM.       

  :
    WORD addr - 
    WORD data - 
    BYTE type - SENSOR_TYPES -  
  ***************************************************************************/
void StoreData(WORD addr, WORD data, BYTE type)
{
    BYTE packet[16];
    BYTE *pData;

    pData = packet; 

  
    ReadRTCC();
    *pData++ = rtcc.byte.year;
    *pData++ = rtcc.byte.mon;
    *pData++ = rtcc.byte.mday;
    *pData++ = rtcc.byte.wday;
    *pData++ = rtcc.byte.hour;
    *pData++ = rtcc.byte.min;
    *pData++ = rtcc.byte.sec;


    *pData++ = type;
    *pData++ = data>>8;
    *pData++ = data;

    writePacket(addr,packet,10);   
    

    while(writeWait())
    {
        IdleUs(EEPROM_ACK_WAIT);
    }

    eeAddress+=0x10; 

    if(eeAddress > 0x7FFF)
    {
        eeAddress = 0x10; 
    }

    StoreAddress();

}


/****************************************************************************
  :
    void StoreSensorData(void)

        EEPROM
  
      SampleSensors()

   -   tempVal, potVal, vddVal, pressedCT1-3
  ***************************************************************************/
void StoreSensorData(void)
{
    #ifdef USE_PWRCTL
        mPWR_ON();
        IdleMs(EEPROM_WAIT);
    #endif

    if(tasks.bits.transmit) 
    {
        _UxMD = 0;
        UARTInit();
        UARTPrintString("Storing Data...\r\n") ;
        _UxMD = 1;
    }

    InitI2C();   
   
    switch(tasks.bits.mode)
    {
        case MODE_TEMP:  

            StoreData(eeAddress,tempVal,TEMP_DATA);
            break;

        case MODE_POT:  

            StoreData(eeAddress,vddVal,VDD_DATA);
            StoreData(eeAddress,potVal,POT_DATA);
            break;

        #ifdef USE_CAPTOUCH
        case MODE_CAP:   

            StoreData(eeAddress,pressedCT1,CT1_DATA);
            #ifndef CT2_IGNORE
            StoreData(eeAddress,pressedCT2,CT2_DATA);
            #endif
            StoreData(eeAddress,pressedCT3,CT3_DATA);
            break;
        #endif

        case MODE_ALL: 

            StoreData(eeAddress,tempVal,TEMP_DATA);
            StoreData(eeAddress,vddVal,VDD_DATA);
            StoreData(eeAddress,potVal,POT_DATA);

            #ifdef USE_CAPTOUCH
                StoreData(eeAddress,pressedCT1,CT1_DATA);
                #ifndef CT2_IGNORE
                StoreData(eeAddress,pressedCT2,CT2_DATA);
                #endif
                StoreData(eeAddress,pressedCT3,CT3_DATA);
            #endif
            break;

        default:

            break;
    }

    CloseI2C();     
    #ifdef USE_PWRCTL
    mPWR_OFF();
    #endif

    tasks.bits.store = 0;   
}//end StoreSensorData()


/****************************************************************************
  :
    void TransmitEEPROM(WORD addrMin, WORD addrMax)

     EEPROM  UART
  
  :
    WORD addrMin -    EEPROM
    WORD addrMax -  
  ***************************************************************************/
void TransmitEEPROM(WORD addrMin,WORD addrMax)
{

    WORD addr=addrMin;
    WORD dataVal;
    WORD temp;
    BYTE readData[16];


    while(addr<addrMax)
    {


        UARTPrintString("Reading...");
        readPacket(addr,readData,10);


        UARTPrintString("Addr: 0x");
        UARTPutHexWord(addr);


        UARTPutChar(' ');
        UARTPutHex(readData[1]);        //
        UARTPutChar('/');
        UARTPutHex(readData[2]);        //
        UARTPrintString("/20");
        UARTPutHex(readData[0]);        //
        UARTPutChar(' ');
        UARTPutHex(readData[3]);        // 
        UARTPutChar(' ');
        UARTPutHex(readData[4]);        //
        UARTPutChar(':');
        UARTPutHex(readData[5]);        //
        UARTPutChar(':');
        UARTPutHex(readData[6]);        //
        UARTPutChar(' ');


        switch(readData[7])
        {
            case POT_DATA:
                UARTPrintString("POT Voltage: ");

                dataVal = (((WORD)readData[8])<<8) + readData[9];

                temp = dataVal/1000;
                UARTPutChar(temp%10 + '0');

                UARTPutChar('.');

                temp = dataVal/100;
                UARTPutChar(temp%10 + '0');

                temp = dataVal/10;
                UARTPutChar(temp%10 + '0');

                temp = dataVal;
                UARTPutChar(temp%10 + '0');
                UARTPutChar('V');
                break;
            case VDD_DATA:
                UARTPrintString("VDD Voltage: ");

                dataVal = (((WORD)readData[8])<<8) + readData[9];

                temp = dataVal/1000;
                UARTPutChar(temp%10 + '0');

                UARTPutChar('.');

                temp = dataVal/100;
                UARTPutChar(temp%10 + '0');

                temp = dataVal/10;
                UARTPutChar(temp%10 + '0');

                temp = dataVal;
                UARTPutChar(temp%10 + '0');
                UARTPutChar('V');
                break;
            case TEMP_DATA:
                UARTPrintString("Temp: ");

                dataVal = (((WORD)readData[8])<<8) + readData[9];

                temp = dataVal/100;
                UARTPutChar(temp%10 + '0');

                temp = dataVal/10;
                UARTPutChar(temp%10 + '0');

                UARTPutChar('.');

                temp = dataVal;
                UARTPutChar(temp%10 + '0');

                UARTPutChar('C');

                break;
            case CT1_DATA:
                UARTPrintString("Touch: ");
                dataVal = (((WORD)readData[8])<<8) + readData[9];
                if(dataVal != 0)
                    UARTPrintString("Cap touch button 1 pressed ");
                else
                    UARTPrintString("Cap touch button 1 unpressed ");
                break;
            case CT3_DATA:
                UARTPrintString("Touch: ");
                dataVal = (((WORD)readData[8])<<8) + readData[9];
                if(dataVal != 0)
                    UARTPrintString("Cap touch button 3 pressed ");
                else
                    UARTPrintString("Cap touch button 3 unpressed ");
                break;
            case CT2_DATA:
                UARTPrintString("Touch: ");
                dataVal = (((WORD)readData[8])<<8) + readData[9];

                if(dataVal != 0)
                    UARTPrintString("Cap touch button 2 pressed ");
                else
                    UARTPrintString("Cap touch button 2 unpressed ");
                break;

            case N0_CT_PRESSED:
                UARTPrintString("Touch: ");
                dataVal = (((WORD)readData[8])<<8) + readData[9];
                UARTPrintString("No button pressed ");
                break;
            default:
                UARTPrintString("Unknown: 0x");

                UARTPutHex(readData[8]);
                UARTPutHex(readData[9]);

                break;
        }

        //Display data
        UARTPutChar('\r');
        UARTPutChar('\n');

        addr+=0x10;
    }
}//end TransmitEEPROM(..)


