#include "system.h"

    _FBS(BSS_OFF & BWRP_OFF)
    _FGS(GCP_OFF & GWRP_OFF)
    _FOSCSEL(FNOSC_FRCDIV & IESO_OFF)
    _FOSC(FCKSM_CSECMD & POSCFREQ_MS & OSCIOFNC_ON & POSCMOD_NONE & SOSCSEL_SOSCLP)
    _FWDT(FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768)
    _FPOR(MCLRE_ON & BORV_LPBOR & BOREN_BOR3 & I2C1SEL_PRI & PWRTEN_OFF)
    _FICD(BKBUG_OFF & ICS_PGx3)
    _FDS(DSWDTEN_OFF & DSBOREN_ON & RTCOSC_SOSC & DSWDTOSC_SOSC & DSWDTPS_DSWDTPSF) //DSWDT SOSC = LPRC


RTCCFORM rtcc;          //RTCC-

WORD eeAddress = 0;     //EEPROM 
WORD potVal;            //POT  mV
WORD tempVal;           //  0.1*C
WORD vddVal;            // 

WORD pressedCT1 = 0;    //1  
WORD pressedCT2 = 0;    //2  
WORD pressedCT3 = 0;    //3  
OSC_MODES osc;
LP_MODES lpMode;        //   
DEMO_TASKS tasks;       // 

WORD potVal_prev=0;            //POT  mV
WORD tempVal_prev=0;           //  0.1*C
WORD vddVal_prev=0; 


int main(void)
{

    WORD_VAL modes;

	osc = OSC_MODE;

    InitIO();      //  I/O
    InitSystem();  // 
    HandleReset(); //  Reset (POR/DPSLP)

    while(1)
    {
	    switch(osc)                  //  
        {
            case MHz_8:
                CLKDIVbits.RCDIV = 0;   //8 MHz clock
                break;

            case MHz_1:
                CLKDIVbits.RCDIV = 3;   //1 MHz clock
                break;

            default:
                break;
        }
        
        GetVddVal();

        if(tasks.bits.button)    //  
        {
            ProcessButtons();       //  
            HandleButtons();        //   
        }

        #ifdef USE_CAPTOUCH
        if ((tasks.bits.mode == MODE_CAP) || (tasks.bits.mode == MODE_ALL)){
 	       if(tasks.bits.cap)       //   
    	    {
        	    GetCapVal();
        	}
        }
        #endif

        if(tasks.bits.sample)    //  
        {
            SampleSensors();
        }

        if(tasks.bits.store)     //  
        {
        	StoreSensorData();
        }

        if(tasks.bits.transmit) //  
        {
			TransmitData();
        }
		
		#ifdef USE_RTCC_ALARM
	        #ifdef USE_CAPTOUCH
		        if((tasks.bits.mode != MODE_CAP) || (tasks.bits.mode != MODE_ALL)) //    -     
    		    {
	        	    _RTCIF = 0;         //   RTCC
    	        	_RTCIE = 1;
        		}

	        #else
		        _RTCIF = 0;         //   RTCC
    		    _RTCIE = 1;
        	#endif
		
        	tasks.bits.alarm = 0;

		#else
			_RTCIE = 0;
		#endif
		

        EnableSW2();            //  
        EnableSW3();			//
     
        switch(lpMode)          //     
        {
            case LP_SLEEP:
                Sleep();        // sleep
                break;

            case LP_DPSLP:

                //     EEPROM
                modes.v[1] = lpMode;
                modes.v[0] = tasks.Val;

                DSGPR1 = modes.Val;

                DSGPR0 = eeAddress;
             
                DSCONbits.DSEN = 1; //deep sleep
                Sleep();


                break;

            case LP_IDLE:
                Idle();         //Idle
                break;

            default:
                break;
        }

        DisableSW2();            //  
        DisableSW3();

    }//end while(1)
}//end main



/****************************************************************************
   I/O:
    void InitIO(void)

  ***************************************************************************/
void InitIO(void)
{
    LATA = 0;           
    LATB = 0;
    TRISA = 0;
    TRISB = 0;

    AD1PCFG = 0xFFFF;  

    POT_AN = 0;        
    TEMP_AN = 0;       
    POT_TRIS = 1; 
    TEMP_TRIS = 1;


    PWRCTL_TRIS = 0;   
    #ifdef USE_PWRCTL
    mPWR_OFF();
    #endif

    TEMP_TRIS = 1;     
    POT_TRIS = 1;      

    SCL_TRIS = 1;
    SDA_TRIS = 1;

    mSetLED2Tris(1);
    mSetLED3Tris(1);

    UTX_LAT = 0;
    UTX_TRIS = 0; 
    URX_TRIS = 1; 
 
}//end InitIO


/****************************************************************************
  :
    void InitSystem(void)

    Reset
  ***************************************************************************/
void InitSystem (void)
{
    //     
    #if defined( OPTIMIZE_A ) || defined( OPTIMIZE_B )
        PMD1 = 0xEFBE;                      //  TMR3,TMR1,UART1,SPI1,  TMR2,I2C1,UART2,ADC
        PMD2 = 0xFFFF;                      //  IC1,OC1
        PMD3 = 0xFDFF;                      //  CMP,CRC,  RTCC
        PMD4 = 0xFFFF;                      //  DataEE,REFO,CTMU,HLVD
    #endif
	
	CLKDIVbits.RCDIV = 0;                   // FRCDIV - 8
    __builtin_write_OSCCONL(OSCCON | 0x02); //  32 
    SRbits.IPL = 2;                         

	tasks.Val = 0;                          //   
    tasks.bits.transmit = DEFAULT_TX;       //   
	
}


/****************************************************************************
  :
    void HandleReset(void)

   Reset
  ***************************************************************************/
void HandleReset(void)
{
    WORD_VAL mode;

    //    Deep Sleep?
    if(RCONbits.DPSLP)
    {
        
        eeAddress = DSGPR0;
        mode.Val = DSGPR1;
        lpMode = mode.v[1];
        tasks.Val = mode.v[0];

        RCONbits.DPSLP = 0;
        DSCONbits.RELEASE = 0;      

        if(tasks.bits.transmit)
        {
            #ifdef USE_PWRCTL
            mPWR_ON();
            IdleMs(UART_WAIT);
            #endif
            _UxMD = 0;
            UARTInit();
            UARTPrintString("Wake from Deep Sleep...");
        }

        switch(ALARM_PERIOD)
        {
            case ONE_SECOND:
                SetRTCCAlarm(1);        //   1 
                break;
            case TEN_SECOND:
                SetRTCCAlarm(2);        //    10 
                break;
            default:
                break;
        }     

        if(DSWAKEbits.DSINT0 != 0)
        {
            if(tasks.bits.transmit)
            {
                UARTPrintString("  INT0\r\n");
            }

            tasks.bits.button = 1;
        }

        if(DSWAKEbits.DSFLT != 0)
        {
            if(tasks.bits.transmit)
            {
                UARTPrintString("  DSFLT\r\n");
            }
        }

        if(DSWAKEbits.DSWDT != 0)
        {
            if(tasks.bits.transmit)
            {
                UARTPrintString("  DSWDTT\r\n");
            }
        }

        if(DSWAKEbits.DSRTC != 0)
        {
            if(tasks.bits.transmit)
            {
                 UARTPrintString("  RTCC\r\n");
            }

            tasks.bits.sample = 1;
        }

        if(DSWAKEbits.DSMCLR != 0)
        {
            if(tasks.bits.transmit)
            {
                UARTPrintString("  MCLR\r\n");
            }
        }

        if(DSWAKEbits.DSPOR != 0)  //POR
        {
            if(tasks.bits.transmit)
            {
                UARTPrintString("  POR\r\n");
            }
        }
        
        #ifdef USE_CAPTOUCH       
 	    // if ((tasks.bits.mode == MODE_CAP) || (tasks.bits.mode == MODE_ALL)){
            InitCapSense();           
            CapSenseTickInit();       
       	 //}  
        #endif
    }
    else     //     MCLR
    {

        #ifdef USE_PWRCTL
        mPWR_ON();
        IdleMs(UART_WAIT);
        #endif

        if(RCONbits.POR)	//  
        {
            InitRTCC();                 //  RTCC
			InitCalendar();				//  
		    if(tasks.bits.transmit)     //  
            {
                _UxMD = 0;
                UARTInit();
                UARTPrintString("nanoWatt XLP board \r\n");
                UARTPrintString("Click S2: Forces a Sample to be taken based on the currently sensor \r\n");
                UARTPrintString("Click S3: Toggles UART on/off. Output of data and the current device state \r\n");
				
		
                UARTPrintString("Hold S2 over 2 sec: switches sensor type\r\n");
                UARTPrintString("Hold S3 over 2 sec: switches low power mode \r\n");
		
				
				UARTPrintString("Hold S2 on reset to clear RTCC and EEPROM data \r\n");
                UARTPrintString("Hold S3 on reset to transmit logged data to PC \r\n");

            }
        }

        if(RCONbits.EXTR)               //MCLR Reset
        {
		
            if(tasks.bits.transmit)    
            {
                _UxMD = 0;
                UARTInit();
                IdleMs(UART_WAIT);
                UARTPrintString("MCLR reset occurred.\r\n\r\n");
            }
        }
    
        SW2_TRIS = 1;
        SW2_PULLUP = 1;                 
        SW3_TRIS = 1;
        SW3_PULLUP = 1;                 
        IdleUs(5);                      

        if(SW2_PORT == 0)      //  Reset  SW2 -    EEPROM
        {                               
            SW2_PULLUP = 0;    
            SW3_PULLUP = 0;    

            eeAddress = 0x10;         
            InitCalendar();

            if(tasks.bits.transmit)
            {
                _UxMD = 0;
                UARTInit();
                UARTPrintString("***Datalog reset!***\n\r");
            }
        }
        else if(SW3_PORT == 0)          //  SW3  Reset,    EEPROM
        {
            SW2_PULLUP = 0;           
            SW3_PULLUP = 0;           

            InitI2C();                  
            GetAddress();     
            GetTimestamp((eeAddress>0x10)?(eeAddress-0x10):0x10);

            
            _UxMD = 0;
            UARTInit();
            UARTPrintString("Transmitting datalog:\n\r");
            TransmitEEPROM(0x10,eeAddress);
            UARTPrintString("Datalog Transmit done!\n\r");

        }
        else                            //   
        {
            SW2_PULLUP = 0;    
            SW3_PULLUP = 0;    

            InitI2C();         
            GetAddress();
            GetTimestamp((eeAddress>0x10)?(eeAddress-0x10):0x10); 
        }

        switch(ALARM_PERIOD)
        {
            case ONE_SECOND:
                SetRTCCAlarm(1);        //  
                break;
            case TEN_SECOND:
                SetRTCCAlarm(2);        //  10 
                break;
            default:
                break;
        }        
		
        lpMode = DEFAULT_MODE;          //    
        tasks.bits.mode = DEFAULT_SENSOR;

        #ifdef USE_CAPTOUCH
        if(tasks.bits.mode > MODE_POT)   
        {
            if(lpMode > LP_SLEEP)       
            {
                lpMode = LP_SLEEP;      

                if(tasks.bits.transmit) 
                {
                    UARTPrintString("Cap Touch buttons not supported in Deep Sleep.\r\n");
                    UARTPrintString("Changing low power mode to Sleep.\r\n");
                }
            }

            InitCapSense();           
            CapSenseTickInit();       
        }
        #endif

        tasks.bits.sample = 1;          //  
        tasks.bits.store = 1;			//  

        RCON=0;                         //  Reset

    }//end if(RCONbits.DPSLP)

    #ifdef USE_PWRCTL
    mPWR_OFF();    
    #endif
    _UxMD = 1;
}//end HandleReset



/****************************************************************************
  :
    void IdleUs(int timeUs)

       Idle
  ***************************************************************************/
void IdleUs(int timeUs)
{
    _T2MD = 0;


    TMR2 = 0;
    PR2 = (GetInstructionClock()/1000000)*timeUs;       //1  = 2 * 500  (4 MHz)


    _T2IE = 1;
    _T2IF = 0;
    _T2IP = 1;
    SRbits.IPL = 2; 
    T2CON = 0x8000;  


    while(!_T2IF)
    {
        Idle();          
    }

 
    _T2IF = 0;
    _T2IE = 0;
    T2CON = 0;
    _T2MD = 1;
}//end IdleUs


/****************************************************************************
  :
    void IdleMs(int timeMs)

      Idle
  ***************************************************************************/
void IdleMs(int timeMs)
{
    _T2MD = 0;
    T2CON = 0x0000;


    TMR2 = 0;
    PR2 = (GetInstructionClock()/64000)*timeMs; //1 mS


    _T2IF = 0;
    _T2IP = 1;
    _T2IE = 1;
    T2CON = 0x8020; 

    //Idle until T2 interrupt occurs
    while(!_T2IF)
    {
        Idle();
    }


    _T2IF = 0;
    _T2IE = 0;
    T2CON = 0;
    _T2MD = 1;

}//end IdleMs


/****************************************************************************
  Function:
    void __attribute__ ((interrupt, shadow,auto_psv)) _RTCCInterrupt(void)

  Summary:
    RTCC interrupt handling routine.

  Description:
    Interrupt on RTCC alarm, indicate RTCC wakeup and Initiate new Sample.

  Precondition:
    InitRTCC();
    InitCalendar();

  Parameters:
    None.

  Returns:
    None.
  ***************************************************************************/
void __attribute__ ((interrupt, shadow,auto_psv)) _RTCCInterrupt(void)
{
    IFS3bits.RTCIF = 0;     //Clear RTCC Alarm interrupt flag
    _RTCIE = 0;             //Disable RTCC interrupt until sleep

    if(tasks.bits.transmit) //Send wakeup source information
    {
        _UxMD = 0;
        UARTInit();
        IdleUs(5);  //Allow UART to stabilize to prevent glitch on first byte
        UARTPrintString("Wake from RTCC\r\n");
         
   	switch(lpMode)  //    
	{
		case LP_SLEEP:
   			UARTPrintString("Current LPMODE: Sleep\r\n");
   			break;

   		case LP_DPSLP:
      		UARTPrintString("Current LPMODE: DeepSleep\r\n");
	  		break;

		case LP_IDLE:
		    UARTPrintString("Current LPMODE: Idle\r\n");
    	   	break;

	   		default:
 	   		break;
	}
	   _UxMD = 1;
    } 
    tasks.bits.alarm = 1;
    tasks.bits.sample = 1;
}


/****************************************************************************
  Function:
    void __attribute__ ((interrupt, shadow,auto_psv)) _T2Interrupt(void)

  Summary:


  Description:


  Precondition:


  Parameters:
    None.

  Returns:
    None.
  ***************************************************************************/
void __attribute__ ((interrupt,auto_psv)) _T2Interrupt()
{
    _T2IF = 0;  //Clear T2 interrupt flag
}


/****************************************************************************
  Function:
    void __attribute__ ((interrupt, shadow,auto_psv)) _INT0Interrupt(void)

  Summary:
    ISR for Pushbutton S2

  Description:
    ISR for S2 button press, Initiates button Processing and sensor Samples.

  Precondition:
    EnableS2()

  Parameters:
    None.

  Returns:
    None.
  ***************************************************************************/
void __attribute__ ((interrupt, shadow,auto_psv)) _INT0Interrupt(void)
{
    _INT0IF = 0;    //Clear INT0 interrupt flag
    _RTCIE = 0;     //disable RTCC interrupt until sleep

    tasks.bits.button = 1;
}


/****************************************************************************
  Function:
    void __attribute__ ((interrupt, shadow,auto_psv)) _CNInterrupt(void)

  Summary:
    ISR for Pushbutton S3

  Description:
    ISR for S3 button press, Initiates button Processing.

  Precondition:
    EnableS3()

  Parameters:
    None.

  Returns:
    None.
  ***************************************************************************/
void __attribute__ ((interrupt, shadow,auto_psv)) _CNInterrupt(void)
{
    _CNIF = 0;      //Clear CN interrupt flag
    _RTCIE = 0;     //disable RTCC interrupt until sleep

    tasks.bits.button = 1;

}


/****************************************************************************
  Function:
    void __attribute__ ((interrupt, shadow,auto_psv)) _T1Interrupt(void)

  Summary:
    ISR for CTMU tick timer

  Description:
    ISR for CTMU tick timer, Initiates cap touch Processing.

  Precondition:
    InitCap()
    TickInit()

  Parameters:
    None.

  Returns:
    None.
  ***************************************************************************/
void  __attribute__((interrupt, shadow, auto_psv)) _T1Interrupt(void)
{
    //Clear flag
    IFS0bits.T1IF = 0;
    _RTCIE = 0;     //disable RTCC interrupt until sleep

    tasks.bits.cap = 1;
}


