#include "system.h"

/****************************************************************************
  :
    void InitRTCC(void)

  RTCC
  ***************************************************************************/
void InitRTCC(void)
{
    _RTCCMD = 0;
    EnableSOSC();                   //  32 
    SetRTCWREN ();                  //   RTCC

    // 
    ALCFGRPT = 0;                   // 
    
    switch(ALARM_PERIOD)
    {
        case ONE_SECOND:
            ALCFGRPTbits.AMASK = 1; //  
            break;
        case TEN_SECOND:
            ALCFGRPTbits.AMASK = 2; //  10 
            break;
        default:
            break;
    }

    ALCFGRPTbits.ARPT = 0;          //   
    ALCFGRPTbits.CHIME = 1;         //  CHIME
    ALCFGRPTbits.ALRMEN = 1 ;       //  

    //  RTCC
    RCFGCALbits.RTCOE = 0;           
    RCFGCALbits.CAL=0;              
    RCFGCALbits.RTCEN = 1;          
    RCFGCALbits.RTCWREN = 0;        

    PADCFG1bits.RTSECSEL = 0b11;    

    //  
    IFS3bits.RTCIF = 0;             //   
    #ifdef USE_RTCC_ALARM
    	IEC3bits.RTCIE = 1;             //    
    #else
    	IEC3bits.RTCIE = 0;
    #endif
    IPC15bits.RTCIP = 4;            //  
}


/****************************************************************************
  :
    void SetRTCCAlarm(BYTE mask)

    
  ***************************************************************************/
void SetRTCCAlarm(BYTE mask)
{
    ALCFGRPTbits.ALRMEN = 0;    //    
    ALCFGRPTbits.AMASK = mask;  //   10   1 
    #ifdef USE_RTCC_ALARM
    	ALCFGRPTbits.ALRMEN = 1 ;   //  
    #endif
}


/****************************************************************************
  :
    void UpdateRTCC(void)
   RTCC 
  ***************************************************************************/
void UpdateRTCC(void)
{

    SetRTCWREN();           //  

    RCFGCALbits.RTCPTR = 0x3;
    RTCVAL=rtcc.w[0];     	//
    RTCVAL=rtcc.w[1];       //
    RTCVAL=rtcc.w[2];       //,  
    RTCVAL=rtcc.w[3];       //, 

    RCFGCALbits.RTCWREN = 0;    //  
}


/****************************************************************************
  :
    void ReadRTCC(void)

	 RTCC
  ***************************************************************************/
void ReadRTCC(void)
{

    RCFGCALbits.RTCPTR = 0b11;    
    rtcc.w[0]=RTCVAL;     
    rtcc.w[1]=RTCVAL;     
    rtcc.w[2]=RTCVAL;     
    rtcc.w[3]=RTCVAL;     
}


/****************************************************************************
  :
    void InitCalendar(void)

   RTCC-
  ***************************************************************************/
void InitCalendar(void)
{
    int dayOfWeek;

    rtcc.byte.sec =  (((__TIME__[6])&0x0F)<<4) | ((__TIME__[7])&0x0F);
    rtcc.byte.min =  (((__TIME__[3])&0x0F)<<4) | ((__TIME__[4])&0x0F);
    rtcc.byte.hour = (((__TIME__[0])&0x0F)<<4) | ((__TIME__[1])&0x0F);
    rtcc.byte.mday = (((__DATE__[4])&0x0F)<<4) | ((__DATE__[5])&0x0F);
    rtcc.byte.year = (((__DATE__[9])&0x0F)<<4) | ((__DATE__[10])&0x0F);

    switch(__DATE__[0])
    {
        case 'J':
            //January, June, or July
            switch(__DATE__[1])
            {
                case 'a':
                    //January
                    rtcc.byte.mon = 0x01;
                    break;
                case 'u':
                    switch(__DATE__[2])
                    {
                        case 'n':
                            //June
                            rtcc.byte.mon = 0x06;
                            break;
                        case 'l':
                            //July
                            rtcc.byte.mon = 0x07;
                            break;
                    }
                    break;
            }
            break;
        case 'F':
            rtcc.byte.mon = 0x02;
            break;
        case 'M':
            //March,May
            switch(__DATE__[2])
            {
                case 'r':
                    //March
                    rtcc.byte.mon = 0x03;
                    break;
                case 'y':
                    //May
                    rtcc.byte.mon = 0x05;
                    break;
            }
            break;
        case 'A':
            //April, August
            switch(__DATE__[1])
            {
                case 'p':
                    //April
                    rtcc.byte.mon = 0x04;
                    break;
                case 'u':
                    //August
                    rtcc.byte.mon = 0x08;
                    break;
            }
            break;
        case 'S':
            rtcc.byte.mon = 0x09;
            break;
        case 'O':
            rtcc.byte.mon = 0x10;
            break;
        case 'N':
            rtcc.byte.mon = 0x11;
            break;
        case 'D':
            rtcc.byte.mon = 0x12;
            break;
    }

    //Calculate day of week based on date
    dayOfWeek = 6;
    dayOfWeek += ((rtcc.byte.year>>4)*10)+(rtcc.byte.year&0x0F);
    dayOfWeek += ((rtcc.byte.year>>4)*10)+(rtcc.byte.year&0x0F)/4;
    switch(rtcc.byte.mon)
    {
        case 0x01:  //January
            //fall through
        case 0x10:  //October
            //add nothing
            break;

        case 0x02:  //February
        case 0x03:  //March
        case 0x11:  //November
            dayOfWeek += 3;
            break;

        case 0x04:  //April
        case 0x07:  //July
            dayOfWeek += 6;
            break;

        case 0x05:  //May
            dayOfWeek += 1;
            break;

        case 0x06:  //June
            dayOfWeek += 4;
            break;

        case 0x08:  //August
            dayOfWeek += 2;
            break;

        case 0x09:  //September
            //fall through
        case 0x12:  //December
            dayOfWeek += 5;
            break;
    }
    dayOfWeek += ((rtcc.byte.mday>>4)*10)+(rtcc.byte.mday&0x0F);
    dayOfWeek = dayOfWeek%7;
    rtcc.byte.wday = dayOfWeek;

    UpdateRTCC();
}


/****************************************************************************
  :
    void GetRTCC(RTCCFORM *rtccVal)
     /
  ***************************************************************************/
void GetRTCC(RTCCFORM* rtccVal)
{

    RCFGCALbits.RTCPTR = 0b11;    
    rtccVal->w[0]=RTCVAL;     
    rtccVal->w[1]=RTCVAL;     
    rtccVal->w[2]=RTCVAL;     
    rtccVal->w[3]=RTCVAL;     
}


