#include "system.h"


/****************************************************************************
  :
    void InitADC(void)

   
  ***************************************************************************/
void InitADC(void)
{
    if(_ADC1MD)
    {
        _ADC1MD = 0;    //  
    
        //  
        AD1PCFG = 0xFFFF;   
        POT_AN = 0;     
        TEMP_AN = 0;    
        TEMP_TRIS = 1;  
        POT_TRIS = 1;
    }

    AD1CON1 = 0x80E0;   // 
    AD1CON2 = 0x0000;   //
    AD1CON3 = 0x0700;   //
}


/****************************************************************************
  :
    WORD GetADCChannel(BYTE channel, BYTE timeTAD)

      
  ***************************************************************************/
WORD GetADCChannel(BYTE channel,BYTE timeTAD)
{
    WORD result;

    InitADC();  // 

    // TAD  
    AD1CON3 = ((WORD)timeTAD<<8) | 0x0000;
    AD1CHS = channel;

    // 
    AD1CON1bits.SAMP = 1;
    _AD1IE = 1;             //   
    _AD1IP = 1;            
    _AD1IF = 0;

    while(!_AD1IF)          //     IDLE
    {
        Idle();
    }

    AD1CON1bits.SAMP = 0;  
    _AD1IF = 0;
    _AD1IE = 0;

    result = ADC1BUF0;

    //   
    #ifdef OPTIMIZE_B
        _ADC1MD = 1;            //     
    #endif

    return result;
}//end GetADCChannel(...)



/****************************************************************************
  :
    void SampleSensors(void)

         
  ***************************************************************************/
void SampleSensors (void)
{

    //   
    if(tasks.bits.transmit)
    {
        #ifdef USE_PWRCTL	//   , 
       		mPWR_ON();
        IdleMs(UART_WAIT);
        #endif
        _UxMD = 0;	//  UART
        UARTInit();	// 
        UARTPrintString("Taking Sample...\r\n") ;
        _UxMD = 1;	// 
    }
    
    GetVddVal();

    //  
    switch(tasks.bits.mode)
    {
        case MODE_TEMP:

            #ifdef USE_PWRCTL		//   
         	   mPWR_ON();			// 
            	IdleMs(TEMP_WAIT);  // 
            #endif

            GetTempVal();           //  
            break;

        case MODE_POT:

            #ifdef USE_PWRCTL
    	        mPWR_ON();
     	       IdleMs(POT_WAIT);       
            #endif

            GetPOTVal();            //   
            break;

        #ifdef USE_CAPTOUCH
        case MODE_CAP:

            GetCapVal();            //  
            break;
        #endif

        case MODE_ALL:

            #ifdef USE_PWRCTL
  	           mPWR_ON();
 	           IdleMs(TEMP_WAIT);     
            #endif

            GetTempVal();           
            GetPOTVal();            

            #ifdef USE_CAPTOUCH
      		     GetCapVal();           
            #endif
            break;

        default:

            break;
    }

    #ifdef USE_PWRCTL
    mPWR_OFF();                     
    #endif

    tasks.bits.store = 1;        //       EEPROM
    tasks.bits.sample = 0;       //   

}//end SampleSensors()


/****************************************************************************
  :
    void GetVddVal(void)

       bandgap-
  ***************************************************************************/
void GetVddVal (void)
{
    //  
    AD1PCFG &= 0x7FFF; 
    IdleUs(100);    //,  

    vddVal = GetADCChannel(VBG_CHANNEL,VBG_TAD); // 

    //  
    AD1PCFG |= 0x8000; 

    vddVal = VBG_VAL/vddVal; // 
}//end GetVddVal


/****************************************************************************
  :
    void GetTempVal(void)

  
  ***************************************************************************/
void GetTempVal (void)
{

#if(TEMP_SENSOR  == MCP9700)

    tempVal = GetADCChannel(TEMP_CHANNEL,TEMP_TAD);     // 
    tempVal = (WORD)((vddVal*(long)tempVal)/1024*10);   // 
                                                        // 
    tempVal = (tempVal - MCP_V0)/MCP_TC;                //  

#elif(TEMP_SENSOR == DIODE)

    GetDiodeTemp(TEMP_CHANNEL);     

    tempVal = (WORD)((vddVal*(long)tempVal)/1024*100);  
                                                        
    tempVal = (DIODE_V0 - tempVal)/DIODE_TC;            


#elif (TEMP_SENSOR > 1)
    #error Please define Tempature sensor
#endif

}//end GetTempVal


/****************************************************************************
  :
    void GetPOTVal(void)

     
  ***************************************************************************/
void GetPOTVal (void)
{
    // 
    potVal = GetADCChannel(POT_CHANNEL,POT_TAD);
    potVal = (WORD)((vddVal*(long)potVal)/1024);  //   
}


/****************************************************************************
  :
    void GetCapVal(void)

     
  ***************************************************************************/
void GetCapVal(void)
{
    BYTE channelSelect;

    //    
    for(channelSelect = 0;channelSelect < NUM_CTMU_CH;channelSelect++)
    {
        #ifdef CT2_IGNORE
        if(channelSelect == 1)
        {
            continue;
        }
        #endif

        ReadCTMU(channelSelect);
        UpdateCTMU(channelSelect);
    }
    
   #ifdef CT2_IGNORE
        pressedCT2 = 0;
   #endif

    //      
    if(pressedCT1 || pressedCT2 || pressedCT3)
    {
        #ifdef USE_PWRCTL
        mPWR_ON();
        #endif
        mSetLED3Tris(0);
        mSetLED3(0);
    }
    else	//    
    {
        mSetLED3(1);
        mSetLED3Tris(1);

    }

    tasks.bits.cap = 0;	//    
}//end GetCapVal





