#include "uart.h"

/*********************************************************************
: char UARTGetChar()

    UART 

********************************************************************/
char UARTGetChar()
{
    char Temp;

    while(_UxRXIF == 0);

    Temp = UxRXREG;
    _UxRXIF = 0;
    return Temp;
}

/*********************************************************************
: void UARTInit(void)

  UART
********************************************************************/
void UARTInit()
{
    UxMODEbits.UARTEN = 1;
    if(CLKDIVbits.RCDIV == 3)		//    1
    {
        UxBRG = BAUDRATEREG/8;	//  
    }
    else
    {
        UxBRG = BAUDRATEREG;	//   8 -  
    }
    UxMODEbits.BRGH = BRGH_VAL;

    #ifdef INVERT_UART
    UxMODEbits.RXINV = 1;  //Invert RX and TX Idle state
    UxSTAbits.UTXINV = 1;
    #endif

    UxSTAbits.UTXEN = 1;


    _UxRXIF = 0;
}

/*******************************************************************************
: UARTIsPressed()

:
    0 :     
    1 :    

*******************************************************************************/
char UARTIsPressed()
{
    if(_UxRXIF == 1)
        return 1;
    return 0;
}

/*******************************************************************************
: UARTPrintString( char *str )

  UART  
*******************************************************************************/
void UARTPrintString( char *str )
{
    unsigned char c;

    while((c = *str++))
    {
        UARTPutChar(c);
    }
}

/*********************************************************************
: void UARTPutChar(char ch)

  UART   ch
********************************************************************/
void UARTPutChar( char ch )
{
    UxTXREG = ch;
    #if !defined(__PIC32MX__)
        Nop();
    #endif
    while(UxSTAbits.TRMT == 0);
}

/*******************************************************************************
: UARTPutDec(unsigned char dec)

        UART
*******************************************************************************/
void  UARTPutDec(unsigned char dec)
{
    unsigned char res;

    res = dec;

    if (res/100)
    {
        UARTPutChar( res/100 + '0' );
    }
    res = res - (res/100)*100;

    if (res/10)
    {
        UARTPutChar( res/10 + '0' );
    }
    res = res - (res/10)*10;

    UARTPutChar( res + '0' );
}

/*******************************************************************************
: UARTPutHex

        UART
*******************************************************************************/

const unsigned char CharacterArray[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

void UARTPutHex( int toPrint )
{
    int printVar;

    printVar = toPrint;
    toPrint = (toPrint>>4) & 0x0F;
    UARTPutChar( CharacterArray[toPrint] );

    toPrint = printVar & 0x0F;
    UARTPutChar( CharacterArray[toPrint] );

    return;
}

/*******************************************************************************
: UARTPutHexWord(unsigned int toPrint)

        UART
*******************************************************************************/
#if defined( __C30__ ) || defined( __PIC32MX__ )
void UARTPutHexWord( unsigned int toPrint )
{
    unsigned int printVar;

    printVar = (toPrint>>12) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>8) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>4) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = toPrint & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    return;
}

void UARTPutHexDWord( unsigned long toPrint )
{
    unsigned long printVar;

    printVar = (toPrint>>28) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>24) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>20) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>16) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>12) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>8) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = (toPrint>>4) & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    printVar = toPrint & 0x0F;
    UARTPutChar( CharacterArray[printVar] );

    return;
}

#endif

/*********************************************************************
: char UARTChar2Hex(char ch)

   ACSII-
********************************************************************/
char UARTChar2Hex(char ch){
  //Wrong char
  if(ch > 102)
    return 0;

  //From a to f
  if(ch > 96)
    return (ch-87);

  //Wrong char
  if(ch > 70)
    return 0;

  //From A to F
  if(ch > 64)
    return (ch-55);

  //Wrong char
  if(ch > 57)
    return 0;

  //From 0 - 9
  if(ch > 47)
    return(ch-48);
  else
  //Wrong char
    return 0;
}

/*********************************************************************
: char UARTHex2Char(char hex)

 ASCII-  
********************************************************************/
char UARTHex2Char(char hex){
char h;
  h = hex&0x0f;
  //0xa - 0xf
  if(h>9)
    return (h+55);
  else
    return (h+48);
}

/*********************************************************************
: void UARTClrError(void)

  
********************************************************************/
void UARTClrError(void){
    //  
    if(UxSTAbits.OERR)
        UxSTAbits.OERR = 0;
}



