#ifndef _UART_H
#define _UART_H


#include "system.h"

//    UART
#ifdef BRGH_VAL
    #define BRG_DIV 4
#else
    #define BRG_DIV 16
#endif

#define BAUDRATEREG    ((FCY + (BRG_DIV/2*BAUDRATE))/BRG_DIV/BAUDRATE-1)
#define BAUD_ACTUAL    (FCY/BRG_DIV/(BAUDRATEREG+1))

#define BAUD_ERROR          ((BAUD_ACTUAL > BAUDRATE) ? BAUD_ACTUAL-BAUDRATE : BAUDRATE-BAUD_ACTUAL)
#define BAUD_ERROR_PRECENT  ((BAUD_ERROR*100+BAUDRATE/2)/BAUDRATE)

#if (BAUD_ERROR_PRECENT > 3)
    #error "UART frequency error is worse than 3%"
#elif (BAUD_ERROR_PRECENT > 2)
    #warning "UART frequency error is worse than 2%"
#endif


#define UARTREG2(a,b)     U##a##b
#define UARTREG(a,b)    UARTREG2(a,b)
#define UARTINT2(a,b)     _U##a##b
#define UARTINT(a,b)    UARTINT2(a,b)

#define UxMODE      UARTREG(UARTNUM,MODE)
#define UxBRG       UARTREG(UARTNUM,BRG)
#define UxSTA       UARTREG(UARTNUM,STA)
#define UxRXREG     UARTREG(UARTNUM,RXREG)
#define UxTXREG     UARTREG(UARTNUM,TXREG)
#define UxMODEbits  UARTREG(UARTNUM,MODEbits)
#define UxSTAbits   UARTREG(UARTNUM,STAbits)
#define _UxMD       UARTINT(UARTNUM,MD)
#define _UxRXIF     UARTINT(UARTNUM,RXIF)
#define _UxTXIF     UARTINT(UARTNUM,TXIF)

#define mFlushUART() while(!UxSTAbits.TRMT)



/*********************************************************************
: char UARTGetChar()

    UART 

********************************************************************/
char UARTGetChar();

/*********************************************************************
: void UARTPutChar(char ch)

  UART   ch
********************************************************************/
void UARTPutChar( char ch );

/*********************************************************************
: void UARTInit(void)

  UART
********************************************************************/
void UARTInit();

/*******************************************************************************
: UARTIsPressed()

:
    0 :     
    1 :    

*******************************************************************************/
char UARTIsPressed();

/*******************************************************************************
: UARTPrintString( char *str )

  UART  
*******************************************************************************/
void UARTPrintString( char *str );

/*******************************************************************************
: UARTPutDec(unsigned char dec)

        UART
*******************************************************************************/
void UARTPutDec( unsigned char dec );

/*******************************************************************************
: UARTPutHex

        UART
*******************************************************************************/
void UARTPutHex( int toPrint );

/*******************************************************************************
: UARTPutHexWord(unsigned int toPrint)

        UART
*******************************************************************************/
#if defined( __C30__ ) || defined( __PIC32MX__ )
void UARTPutHexWord( unsigned int toPrint );
void UARTPutHexDWord( unsigned long int toPrint );
#endif

/*********************************************************************
: char UARTChar2Hex(char ch)

   ACSII-
********************************************************************/
char UARTChar2Hex(char ch);

/*********************************************************************
: char UARTHex2Char(char hex)

 ASCII-  
********************************************************************/
char UARTHex2Char(char hex);

/*********************************************************************
: void UARTClrError(void)

  
********************************************************************/
void UARTClrError(void);

/*********************************************************************
: UARTDataReceived()

  
********************************************************************/
#define UARTDataReceived() (UxSTAbits.URXDA)

#endif


